@@ -13,23 +13,27 @@ extremely handy for recovering the syntactic structure of actual rules, so that'
 
 """
 import re, os, collections
-from typing import Optional, NamedTuple, List
-from ..support import foundation, compaction, interfaces, failureprone
-from ..parsing import automata
-from ..scanning import finite, regular, miniscan
-from . import grammar
+from typing import NamedTuple, List
+from ..support import foundation, failureprone
+from ..parsing.context_free import ContextFreeGrammar
+from ..parsing.automata import DragonBookTable, ParsingStyle, GeneralizedStyle, DeterministicStyle, HFA, tabulate
+from ..parsing.all_methods import PARSE_TABLE_METHODS
+from ..scanning import finite, regular, charset
+from ..scanning.interface import INITIAL
+from . import grammar, compaction
+from .interface import ScanAction
 
 
 class TextBookForm:
 	""" This provides the various views of the text-book form of scan and parse tables. """
-	def __init__(self, *, dfa: finite.DFA, scan_actions:List[interfaces.ScanAction], parse_table: automata.DragonBookTable):
+	def __init__(self, *, dfa: finite.DFA, scan_actions:List[ScanAction], parse_table: DragonBookTable):
 		self.dfa = dfa
 		self.scan_actions = scan_actions
 		self.parse_table = parse_table
 	def as_compact_form(self, *, filename):
 		return {
 			'description': 'MacroParse Automaton',
-			'version': (0, 0, 2),
+			'version': (0, 0, 3),
 			'source': filename,
 			'scanner': self.compact_scanner(),
 			'parser': self.compact_parser(),
@@ -39,7 +43,7 @@ class TextBookForm:
 		if dfa is None: return
 		return {
 			'dfa': compaction.compress_scanner(initial=dfa.initial, matrix=dfa.states, final=dfa.final),
-			'action': dict(zip(interfaces.ScanAction._fields, zip(*self.scan_actions))),
+			'action': dict(zip(ScanAction._fields, zip(*self.scan_actions))),
 			'alphabet': {'bounds': dfa.alphabet.bounds, 'classes': dfa.alphabet.classes,}
 		}
 	def compact_parser(self):
@@ -72,12 +76,13 @@ class TextBookForm:
 
 class IntermediateForm(NamedTuple):
 	nfa: finite.NFA
-	scan_actions: List[interfaces.ScanAction]
-	hfa: automata.HFA
-	parse_style:automata.ParsingStyle
+	scan_actions: List[ScanAction]
+	hfa: HFA
+	cfg: ContextFreeGrammar
+	parse_style:ParsingStyle
 	def determinize(self) -> TextBookForm:
 		dfa = self.nfa.subset_construction().minimize_states().minimize_alphabet() if self.nfa.states else None
-		return TextBookForm(dfa=dfa, scan_actions=self.scan_actions, parse_table=automata.tabulate(self.hfa, style=self.parse_style))
+		return TextBookForm(dfa=dfa, scan_actions=self.scan_actions, parse_table=tabulate(self.hfa, self.cfg, style=self.parse_style))
 	def make_dot_file(self, path): self.hfa.make_dot_file(path)
 
 
@@ -85,17 +90,18 @@ def compile_string(document:str, *, method="LR1") -> IntermediateForm:
 	text = failureprone.SourceText(document)
 	return _compile_text(text, method=method)
 
-def compile_file(pathname, *, method, strict=False) -> dict:
+def compile_file(pathname, *, method, verbose=False) -> dict:
 	filename = os.path.basename(pathname)
 	with(open(pathname)) as fh: text = failureprone.SourceText(fh.read(), filename=filename)
-	return _compile_text(text, method=method).determinize().as_compact_form(filename=filename)
-
+	intermediate_form = _compile_text(text, method=method)
+	textbook_form = intermediate_form.determinize()
+	if verbose:
+		print("\n  -- ", pathname, " --")
+		textbook_form.pretty_print()
+	return textbook_form.as_compact_form(filename=filename)
 
 STRERROR = {
-	regular.BadReferenceError: "The name {0} is not defined.",
-	regular.CircularDefinitionError: "The name {0} is involved in a circular definition.",
-	regular.NonClassError: "The name {0} is used in a character class, but is not defined as a character (class).",
-	regular.TrailingContextError: "Variable size for both stem and trailing context is not currently supported.",
+	regular.VariableTrailingContextError: "Variable size for both stem and trailing context is not currently supported.",
 }
 
 def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateForm:
@@ -111,12 +117,8 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 			raise grammar.DefinitionError('At line %d: Malformed pattern.' % line_number) from None
 
 	def definitions():
-		name, regex = current_line_text.split(None, 1)
-		if name in env: raise grammar.DefinitionError('You cannot redefine named subexpression %r at line %d.'%(name, line_number))
-		if not re.fullmatch(r'[A-Za-z][A-Za-z_]+', name): raise grammar.DefinitionError('Subexpression %r ought to obey the rule at line %d.'%(name, line_number))
-		try: env[name] = miniscan.rex.parse(miniscan.META.scan(regex), language='Regular')
-		except Exception as e: handle_meta_exception(e, regex)
-		assert env[name].symbol.category in ('char_class', 'regex'), "This would be a bug: got a %r."%env[name].symbol.label
+		name, subexpression = current_line_text.split(None, 1)
+		regular.let_subexpression(env, name, subexpression)
 	
 	def conditions():
 		"""
@@ -131,10 +133,10 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 
 	def note_pattern(pattern_text):
 		# Now patterns that share a trail length can also share a rule ID number.
-		try: bol, expression, trail = miniscan.analyze_pattern(pattern_text, env)
+		try: rule_pattern = regular.analyze_pattern(pattern_text, env)
 		except regular.PatternError as e:
 			handle_meta_exception(e, pattern_text)
-		else: pending_patterns[trail].append((bol, expression))
+		else: pending_patterns[rule_pattern.trail_code].append(rule_pattern)
 
 	def patterns():
 		# This could be done better: a nice exercise might be to enhance the present regex parser to also
@@ -150,23 +152,27 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 			m = re.fullmatch(r'(.*?)\s*:([A-Za-z][A-Za-z_]*)(?:\s+([A-Za-z_]+))?(?:\s+:(0|[1-9][0-9]*))?', current_line_text)
 			if not m: raise grammar.DefinitionError('Unable to analyze overall pattern/action/argument/(rank) structure at line %d.'%line_number)
 			pattern_text, action, argument, rank_string = m.groups()
+			message = [action]
+			if argument is not None: message.append(argument)
 			rank = int(rank_string) if rank_string else 0
 			note_pattern(pattern_text)
-			for trail, list_of_patterns in pending_patterns.items():
-				rule_id = foundation.allocate(scan_actions, interfaces.ScanAction(line_number, trail, action, argument))
-				for bol, expression in list_of_patterns:
-					src = nfa.new_node(rank)
+			for trail_code, list_of_patterns in pending_patterns.items():
+				rule_id = foundation.allocate(scan_actions, ScanAction(trail_code, message, line_number))
+				for rule_pattern in list_of_patterns:
 					dst = nfa.new_node(rank)
-					for q,b in zip(nfa.condition(current_pattern_group), bol):
-						if b: nfa.link_epsilon(q, src)
 					nfa.final[dst] = rule_id
-					expression.tour(regular.Encoder(nfa, rank, env), src, dst)
+					encoder = regular.Encoder(nfa, annotation=rule_pattern.annotation, rank=rank)
+					src = encoder(rule_pattern.tree, dst)
+					for q,b in zip(nfa.condition(current_pattern_group), rule_pattern.bol):
+						if b: nfa.link_epsilon(q, src)
 			pending_patterns.clear()
 		pass
 	
-	def precedence(): ebnf.read_precedence_line(current_line_text, line_number)
+	def precedence():
+		ebnf.read_precedence_line(current_line_text, line_number)
 	
-	def productions(): ebnf.read_production_line(current_line_text, line_number)
+	def productions():
+		ebnf.read_production_line(current_line_text, line_number)
 	
 	def decide_section():
 		# Looks at a header line to see which parsing mode/section to shift into based on a leading keyword,
@@ -183,7 +189,7 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 			return conditions
 		if head == 'patterns':
 			nonlocal current_pattern_group
-			current_pattern_group = tokens[1] if len(tokens)>1 else interfaces.DEFAULT_INITIAL_CONDITION
+			current_pattern_group = tokens[1] if len(tokens)>1 else INITIAL
 			return patterns
 		if head in ('precedence', 'declarations'): return precedence
 		if head == 'productions':
@@ -199,7 +205,7 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 	ebnf = grammar.EBNF_Definition(error_help)
 	
 	# The regular (finite-state) portion of the definition:
-	env = miniscan.PRELOAD['ASCII'].copy()
+	env = charset.mode_normal.new_child(document.filename or "text")
 	nfa = finite.NFA()
 	pending_patterns = collections.defaultdict(list) # Those awaiting an application of the `|` action...
 	scan_actions = [] # That of a regular-language rule entry is <message, parameter, trail, line_number>
@@ -239,13 +245,13 @@ def _compile_text(document:failureprone.SourceText, *, method) -> IntermediateFo
 	
 	# Compose the control tables. (Compaction is elsewhere. Serialization will be straight JSON via standard library.)
 	if condition_definitions: tie_conditions()
-	hfa = automata.PARSE_TABLE_METHODS[method](ebnf.sugarless_form())
-	if ebnf.nondeterministic_symbols:
-		style = automata.GeneralizedStyle(len(hfa.graph), ebnf.nondeterministic_symbols)
+	cfg = ebnf.sugarless_form()
+	hfa = PARSE_TABLE_METHODS[method](cfg)
+	if ebnf.is_nondeterministic:
+		style = GeneralizedStyle(len(hfa.graph))
 	else:
-		style = automata.DeterministicStyle(False)
-	return IntermediateForm(nfa=nfa, scan_actions=scan_actions, hfa=hfa, parse_style=style,)
-
+		style = DeterministicStyle(False)
+	return IntermediateForm(nfa=nfa, scan_actions=scan_actions, hfa=hfa, cfg=cfg, parse_style=style,)
 
 def encode_parse_rules(rules:list, constructors:list, origins:list) -> dict:
 	assert isinstance(rules, list), type(rules)
